//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
#include "M2000.h"
#include <dir.h>
#include <vcl/inifiles.hpp>
#include <process.h>
#include <stdio.h>

#define Section	"M2000"		// ini file section

static AnsiString &GetShortPath(AnsiString &LongPath)
{
 char buf[1024];
 static AnsiString ShortPath;
 strcpy(buf,LongPath.c_str());
 GetShortPathName(LongPath.c_str(),buf,sizeof(buf));
 ShortPath=AnsiString(buf);
 return ShortPath;
}

TM2000::TM2000() :
 FVideoMode(1),
 FP2000Model(0),
 FPrinterType(0),
 FVerbose(1),
 FSync(1),
 FBoot(0),
 FJoystick(1),
 FRAMSize(32),
 FUPeriod(1),
 FIFreq(50),
 FCPUSpeed(100),
 FSoundDevice(255),
 FVolume(10),
 FM2000Path(""),
 FCassetteImage(""),
 FMonitorROM(""),
 FCartridgeROM(""),
 FFontImage(""),
 FPrinterLog("")
{
 TempFile[0]='\0';
}

TM2000::~TM2000()
{
 // Remove temp file if it's still there
 if (TempFile[0]) remove(TempFile);
}

void TM2000::LoadFromFile(AnsiString &file)
{
 TIniFile &iniFile=*new TIniFile(file);
 VideoMode=iniFile.ReadInteger(Section,"VideoMode",1);
 P2000Model=iniFile.ReadInteger(Section,"P2000Model",0);
 PrinterType=iniFile.ReadInteger(Section,"PrinterType",0);
 Verbose=iniFile.ReadInteger(Section,"Verbose",1);
 Sync=iniFile.ReadInteger(Section,"Sync",1);
 Boot=iniFile.ReadInteger(Section,"Boot",0);
 Joystick=iniFile.ReadInteger(Section,"Joystick",1);
 RAMSize=iniFile.ReadInteger(Section,"RAMSize",32);
 UPeriod=iniFile.ReadInteger(Section,"UPeriod",1);
 IFreq=iniFile.ReadInteger(Section,"IFreq",50);
 CPUSpeed=iniFile.ReadInteger(Section,"CPUSpeed",100);
 SoundDevice=iniFile.ReadInteger(Section,"SoundDevice",255);
 Volume=iniFile.ReadInteger(Section,"Volume",10);
 M2000Path=iniFile.ReadString(Section,"M2000Path","");
 CassetteImage=iniFile.ReadString(Section,"CassetteImage","");
 MonitorROM=iniFile.ReadString(Section,"MonitorROM","");
 CartridgeROM=iniFile.ReadString(Section,"CartridgeROM","");
 FontImage=iniFile.ReadString(Section,"FontImage","");
 PrinterLog=iniFile.ReadString(Section,"PrinterLog","");
 UserOptions=iniFile.ReadString(Section,"UserOptions","");
 delete &iniFile;
}

void TM2000::SaveToFile(AnsiString &file)
{
 TIniFile &iniFile=*new TIniFile(file);
 iniFile.WriteInteger(Section,"VideoMode",VideoMode);
 iniFile.WriteInteger(Section,"P2000Model",P2000Model);
 iniFile.WriteInteger(Section,"PrinterType",PrinterType);
 iniFile.WriteInteger(Section,"Verbose",Verbose);
 iniFile.WriteInteger(Section,"Sync",Sync);
 iniFile.WriteInteger(Section,"Boot",Boot);
 iniFile.WriteInteger(Section,"Joystick",Joystick);
 iniFile.WriteInteger(Section,"RAMSize",RAMSize);
 iniFile.WriteInteger(Section,"UPeriod",UPeriod);
 iniFile.WriteInteger(Section,"IFreq",IFreq);
 iniFile.WriteInteger(Section,"CPUSpeed",CPUSpeed);
 iniFile.WriteInteger(Section,"SoundDevice",SoundDevice);
 iniFile.WriteInteger(Section,"Volume",Volume);
 iniFile.WriteString(Section,"M2000Path",M2000Path);
 iniFile.WriteString(Section,"CassetteImage",CassetteImage);
 iniFile.WriteString(Section,"MonitorROM",MonitorROM);
 iniFile.WriteString(Section,"CartridgeROM",CartridgeROM);
 iniFile.WriteString(Section,"FontImage",FontImage);
 iniFile.WriteString(Section,"PrinterLog",PrinterLog);
 iniFile.WriteString(Section,"UserOptions",UserOptions);
 delete &iniFile;
}

void TM2000::GetEmulatorOptions (AnsiString &s)
{
 s="";
 s+="-video "+AnsiString(FVideoMode)+"\r\n";
 if (FP2000Model) s+="-m\r\n";
 else s+="-t\r\n";
 s+="-printertype "+AnsiString(FPrinterType)+"\r\n";
 s+="-verbose "+AnsiString(FVerbose)+"\r\n";
 s+="-sync "+AnsiString(FSync)+"\r\n";
 s+="-boot "+AnsiString(FBoot)+"\r\n";
 s+="-joystick "+AnsiString(FJoystick)+"\r\n";
 s+="-ram "+AnsiString(FRAMSize)+"\r\n";
 s+="-uperiod "+AnsiString(FUPeriod)+"\r\n";
 s+="-ifreq "+AnsiString(FIFreq)+"\r\n";
 s+="-cpuspeed "+AnsiString(FCPUSpeed)+"\r\n";
 s+="-sound "+AnsiString(FSoundDevice)+"\r\n";
 s+="-volume "+AnsiString(FVolume)+"\r\n";
 if (FCassetteImage!="")
  s+="-tape "+AnsiString(GetShortPath(FCassetteImage))+"\r\n";
 if (FMonitorROM!="")
  s+="-romfile "+AnsiString(GetShortPath(FMonitorROM))+"\r\n";
 if (FFontImage!="")
  s+="-font "+AnsiString(GetShortPath(FFontImage))+"\r\n";
 if (FPrinterLog!="")
  s+="-printer "+AnsiString(GetShortPath(FPrinterLog))+"\r\n";
 if (FCartridgeROM!="")
  s+=AnsiString(GetShortPath(FCartridgeROM))+"\r\n";
 if (FUserOptions!="")
 {
  s+="\r\n"+FUserOptions+"\r\n";
 }
}

void TM2000::SetVideoMode(int videoMode)
{
 FVideoMode=(videoMode)? 1:0;
}

void TM2000::SetP2000Model(int p2000Model)
{
 FP2000Model=(p2000Model)? 1:0;
}

void TM2000::SetPrinterType(int printerType)
{
 FPrinterType=(printerType)? 1:0;
}

void TM2000::SetVerbose(int verbose)
{
 FVerbose=(verbose)? 1:0;
}

void TM2000::SetSync(int sync)
{
 FSync=(sync)? 1:0;
}

void TM2000::SetBoot(int boot)
{
 FBoot=(boot)? 1:0;
}

void TM2000::SetJoystick(int joystick)
{
 FJoystick=(joystick)? 1:0;
}

void TM2000::SetRAMSize(int ramSize)
{
 if (ramSize<16) ramSize=16;
 if (ramSize>9999) ramSize=9999;
 FRAMSize=ramSize;
}

void TM2000::SetUPeriod(int uPeriod)
{
 if (uPeriod<1) uPeriod=1;
 if (uPeriod>9) uPeriod=9;
 FUPeriod=uPeriod;
}

void TM2000::SetIFreq(int iFreq)
{
 if (iFreq<10) iFreq=10;
 if (iFreq>500) iFreq=999;
 FIFreq=iFreq;
}

void TM2000::SetCPUSpeed(int cpuSpeed)
{
 if (cpuSpeed<10) cpuSpeed=10;
 if (cpuSpeed>999) cpuSpeed=999;
 FCPUSpeed=cpuSpeed;
}

void TM2000::SetSoundDevice(int soundDevice)
{
 if (soundDevice<0) soundDevice=0;
 if (soundDevice>2) soundDevice=255;
 FSoundDevice=soundDevice;
}

void TM2000::SetVolume(int volume)
{
 if (volume<0) volume=0;
 if (volume>15) volume=15;
 FVolume=volume;
}

void TM2000::SetM2000Path(AnsiString &m2000Path)
{
 FM2000Path=m2000Path;
 char s[MAXPATH];
 char drive[MAXDRIVE];
 char dir[MAXDIR];
 fnsplit (FM2000Path.c_str(),drive,dir,NULL,NULL);
 fnmerge (s,drive,dir,"M2000",".txt");
 FReadMeFile=AnsiString(s);
}

void TM2000::SetCassetteImage(AnsiString &cassetteImage)
{
 FCassetteImage=cassetteImage;
}

void TM2000::SetMonitorROM(AnsiString &monitorROM)
{
 FMonitorROM=monitorROM;
}

void TM2000::SetCartridgeROM(AnsiString &cartridgeROM)
{
 FCartridgeROM=cartridgeROM;
}

void TM2000::SetFontImage(AnsiString &fontImage)
{
 FFontImage=fontImage;
}

void TM2000::SetPrinterLog(AnsiString &printerLog)
{
 FPrinterLog=printerLog;
}

void TM2000::SetUserOptions(AnsiString &userOptions)
{
 FUserOptions=userOptions;
}

int TM2000::Execute(void)
{
 char buf[MAXPATH+1];
 // If an old temp file still exists, remove it before creating a new one
 if (TempFile[0]) remove(TempFile);
 // Get a temporary file name
 GetTempPath(sizeof(buf),buf);
 GetTempFileName(buf,"m2b",0,TempFile);
 // Get emulator options
 AnsiString ansiString;
 GetEmulatorOptions(ansiString);
 // Open the temp file and put emulator options in it
 FILE *file=fopen(TempFile,"wb");
 if (file)
  if (fwrite(ansiString.c_str(),1,ansiString.Length(),file)!=(size_t)ansiString.Length())
  {
   fclose (file);
   remove (TempFile);
   file=NULL;
  }
 if (!file)
 {
  TempFile[0]='\0';
  return -1;
 }
 fclose (file);
 // Spawn M2000. Note the temp file isn't deleted afterwards as we want it to be
 // alive when M2000 is initialising. Instead, we delete it at the next call to
 // this function or when the application terminates
 sprintf (buf,"@%s",GetShortPath(AnsiString(TempFile)).c_str());
 int i=spawnl(P_NOWAIT,M2000Path.c_str(),M2000Path.c_str(),buf,NULL);
 return (i==-1)? errno:0;
}

