//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
#include <dir.h>
#include <vcl/inifiles.hpp>
#include <ctype.h>
#include <stdio.h>
#include <process.h>
#include <errno.h>
#include <stdlib.h>

#include "M2000BuilderMain.h"
#include "M2000BuilderAboutBox.h"
#include "M2000ReadMe.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
#pragma codestring "This executable copyright (C) M. de Kogel, 1997"

#define Section	"Builder"		// ini file section

TM2000BuilderMainForm *M2000BuilderMainForm;
//---------------------------------------------------------------------------
__fastcall TM2000BuilderMainForm::TM2000BuilderMainForm(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::AboutButtonClick(TObject *Sender)
{
 TAboutBox *aboutBox=new TAboutBox(this);
 aboutBox->ShowModal();
 delete aboutBox;
}
//---------------------------------------------------------------------------

void __fastcall TM2000BuilderMainForm::UPeriodUpDownClick(TObject *Sender, TUDBtnType Button)
{
 (Button==btNext)? ++M2000->UPeriod:--M2000->UPeriod;
 UPeriodEdit->Text=AnsiString(M2000->UPeriod);
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::VolumeUpDownClick(TObject *Sender, TUDBtnType Button)
{
 (Button==btNext)? ++M2000->Volume : --M2000->Volume;
 VolumeEdit->Text=AnsiString(M2000->Volume);
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::RAMSizeUpDownClick(TObject *Sender, TUDBtnType Button)
{
 (Button==btNext)? M2000->RAMSize+=16:M2000->RAMSize-=16;
 RAMSizeEdit->Text=AnsiString(M2000->RAMSize);
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::IFreqUpDownClick(TObject *Sender, TUDBtnType Button)
{
 (Button==btNext)? ++M2000->IFreq:--M2000->IFreq;
 IFreqEdit->Text=AnsiString(M2000->IFreq);
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::CPUSPeedUpDownClick(TObject *Sender, TUDBtnType Button)
{
 (Button==btNext)? ++M2000->CPUSpeed:--M2000->CPUSpeed;
 CPUSpeedEdit->Text=AnsiString(M2000->CPUSpeed);
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::FormCreate(TObject *Sender)
{
 char s[MAXPATH];
 char drive[MAXDRIVE];
 char dir[MAXDIR];
 fnsplit (Application->ExeName.c_str(),drive,dir,NULL,NULL);
 fnmerge (s,drive,dir,"M2000Builder",".txt");
 ReadMeFile=AnsiString(s);
 fnmerge (s,drive,dir,"M2000Builder",".ini");
 IniFile=AnsiString(s);
 try
 {
  BuilderReadMeMemo->Lines->LoadFromFile(ReadMeFile);
 }
 catch (Exception &exception)
 {
  BuilderReadMeMemo->Lines->Text="Cannot find M2000Builder.txt";
 }
 M2000=new TM2000;
 M2000->LoadFromFile(IniFile);
 TIniFile &iniFile=*new TIniFile(IniFile);
 AnsiString &a=iniFile.ReadString (Section,"cwd","");
 if (a.c_str()[0])
 {
  chdir (a.c_str());
  setdisk (tolower(a.c_str()[0])-'a');
 }
 else
  AboutButtonClick(this);
 delete &iniFile;
 SetAllOptions ();
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::FormDestroy(TObject *Sender)
{
 UpdateAllOptions();
 M2000->SaveToFile(IniFile);
 delete M2000;
 TIniFile &iniFile=*new TIniFile(IniFile);
 char s[MAXPATH];
 getcwd (s,sizeof(s));
 iniFile.WriteString (Section,"cwd",AnsiString(s));
 delete &iniFile;
}
//---------------------------------------------------------------------------
void TM2000BuilderMainForm::SetAllOptions(void)
{
 VideoModeRadioGroup->ItemIndex=M2000->VideoMode;
 P2000ModelRadioGroup->ItemIndex=M2000->P2000Model;
 PrinterTypeRadioGroup->ItemIndex=M2000->PrinterType;
 VerboseCheckBox->Checked=M2000->Verbose;
 SyncCheckBox->Checked=M2000->Sync;
 BootCheckBox->Checked=M2000->Boot;
 JoystickCheckBox->Checked=M2000->Joystick;
 RAMSizeEdit->Text=AnsiString(M2000->RAMSize);
 UPeriodEdit->Text=AnsiString(M2000->UPeriod);
 IFreqEdit->Text=AnsiString(M2000->IFreq);
 CPUSpeedEdit->Text=AnsiString(M2000->CPUSpeed);
 SoundRadioGroup->ItemIndex=(M2000->SoundDevice==255)? 0:M2000->SoundDevice+1;
 VolumeEdit->Text=AnsiString(M2000->Volume);
 M2000PathEdit->Text=M2000->M2000Path;
 CassetteEdit->Text=M2000->CassetteImage;
 MonitorEdit->Text=M2000->MonitorROM;
 CartridgeEdit->Text=M2000->CartridgeROM;
 FontEdit->Text=M2000->FontImage;
 PrinterLogEdit->Text=M2000->PrinterLog;
 UserOptionsMemo->Text=M2000->UserOptions;
 SetEmulatorOptions();
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::RadioGroupClick(TObject *Sender)
{
 TRadioGroup *radioGroup=(TRadioGroup*)Sender;
 int index=radioGroup->ItemIndex;
 if (Sender==VideoModeRadioGroup)
  M2000->VideoMode=index;
 else if (Sender==P2000ModelRadioGroup)
  M2000->P2000Model=index;
 else if (Sender==PrinterTypeRadioGroup)
  M2000->PrinterType=index;
 else if (Sender==SoundRadioGroup)
  M2000->SoundDevice=(index)? index-1 : 255;
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::MainTabbedNotebookChange(TObject *Sender,
	int NewTab, bool &AllowChange)
{
 static int OldTab=0;
 if (OldTab==0)
 {
  StringEditExit (M2000PathEdit);
  StringEditExit (CassetteEdit);
  StringEditExit (MonitorEdit);
  StringEditExit (CartridgeEdit);
  StringEditExit (FontEdit);
  StringEditExit (PrinterLogEdit);
 }
 else if (OldTab==1)
 {
  EditExit (RAMSizeEdit);
  EditExit (UPeriodEdit);
  EditExit (IFreqEdit);
  EditExit (CPUSpeedEdit);
  EditExit (VolumeEdit);
 }
 else if (OldTab==2)
  MemoExit (UserOptionsMemo);
 if (NewTab==3)
  SetEmulatorOptions();
 else if (NewTab==4)
 {
  try
  {
   M2000ReadMeMemo->Lines->LoadFromFile(M2000->ReadMeFile);
  }
  catch (Exception &exception)
  {
   M2000ReadMeMemo->Lines->Text="Cannot find M2000.txt";
  }
 }
 OldTab=NewTab;
}
//---------------------------------------------------------------------------
void TM2000BuilderMainForm::SetEmulatorOptions(void)
{
 AnsiString *s=new AnsiString;
 M2000->GetEmulatorOptions(*s);
 OptionsMemo->Text=*s;
 delete s;
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::CheckBoxClick(TObject *Sender)
{
 TCheckBox *checkBox=(TCheckBox*)Sender;
 bool checked=checkBox->Checked;
 if (Sender==VerboseCheckBox)
  M2000->Verbose=checked;
 else if (Sender==BootCheckBox)
  M2000->Boot=checked;
 else if (Sender==JoystickCheckBox)
  M2000->Joystick=checked;
 else if (Sender==SyncCheckBox)
  M2000->Sync=checked;
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::EditExit(TObject *Sender)
{
 TEdit *edit=(TEdit*)Sender;
 int value=atoi(edit->Text.c_str());
 if (Sender==RAMSizeEdit)
 {
  M2000->RAMSize=value;
  value=M2000->RAMSize;
 }
 else if (Sender==UPeriodEdit)
 {
  M2000->UPeriod=value;
  value=M2000->UPeriod;
 }
 else if (Sender==IFreqEdit)
 {
  M2000->IFreq=value;
  value=M2000->IFreq;
 }
 else if (Sender==CPUSpeedEdit)
 {
  M2000->CPUSpeed=value;
  value=M2000->CPUSpeed;
 }
 else if (Sender==VolumeEdit)
 {
  M2000->Volume=value;
  value=M2000->Volume;
 }
 edit->Text=AnsiString(value);
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::MemoExit(TObject *Sender)
{
 if (Sender==UserOptionsMemo)
  M2000->UserOptions=UserOptionsMemo->Text;
}
//---------------------------------------------------------------------------
void TM2000BuilderMainForm::UpdateAllOptions (void)
{
 bool b;
 for (int i=0;i<=5;++i)
  MainTabbedNotebookChange(this,i,&b);
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::LaunchButtonClick(TObject *Sender)
{
 // First, update all options
 UpdateAllOptions ();
 // Now spawn M2000
 int i=M2000->Execute();
 // And parse error codes
 if (i)
 {
  AnsiString ansiString="Cannot execute M2000.exe:\r\n";
  switch (i)
  {
   case -1:
    ansiString+="Cannot create temporary file";
    break;
   case E2BIG:
    ansiString+="Arg list too long";
    break;
   case EINVAL:
    ansiString+="Invalid argument";
    break;
   case ENOENT:
    ansiString+="Path or file name not found";
    break;
   case ENOEXEC:
    ansiString+="Exec format error";
    break;
   case	ENOMEM:
    ansiString+="Not enough memory";
    break;
   default:
    ansiString+="Unkown error";
    break;
  }
  Application->MessageBox(ansiString.c_str(),"Error",MB_OK);
 }
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::StringEditExit(TObject *Sender)
{
 TEdit *edit=(TEdit*)Sender;
 AnsiString &s=edit->Text;
 if (Sender==M2000PathEdit)
  M2000->M2000Path=s;
 else if (Sender==CassetteEdit)
  M2000->CassetteImage=s;
 else if (Sender==MonitorEdit)
  M2000->MonitorROM=s;
 else if (Sender==CartridgeEdit)
  M2000->CartridgeROM=s;
 else if (Sender==FontEdit)
  M2000->FontImage=s;
 else if (Sender==PrinterLogEdit)
  M2000->PrinterLog=s;
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::LaunchBitBtnClick(TObject *Sender)
{
 TReadMeForm *readMeForm=new TReadMeForm(this);
 if (Sender==LaunchBuilderReadMeBitBtn)
 {
  readMeForm->Caption="M2000 Builder Read Me";
  readMeForm->ReadMeMemo->Lines=BuilderReadMeMemo->Lines;
 }
 else if (Sender==LaunchM2000ReadMeBitBtn)
 {
  readMeForm->Caption="M2000 Read Me";
  readMeForm->ReadMeMemo->Lines=M2000ReadMeMemo->Lines;
 }
 readMeForm->ShowModal();
 delete readMeForm;
}
//---------------------------------------------------------------------------
void __fastcall TM2000BuilderMainForm::OpenButtonClick(TObject *Sender)
{
 if (Sender==M2000PathButton)
 {
  OpenDialogue->DefaultExt="exe";
  OpenDialogue->FileName=M2000->M2000Path;
  OpenDialogue->Filter="Applications|*.exe";
 }
 else if (Sender==CassetteButton)
 {
  OpenDialogue->DefaultExt="cas";
  OpenDialogue->FileName=M2000->CassetteImage;
  OpenDialogue->Filter="Cassette images|*.cas";
 }
 else if (Sender==MonitorButton || Sender==CartridgeButton)
 {
  OpenDialogue->DefaultExt="bin";
  if (Sender==MonitorButton)
   OpenDialogue->FileName=M2000->MonitorROM;
  else
   OpenDialogue->FileName=M2000->CartridgeROM;
  OpenDialogue->Filter="Binary images|*.bin";
 }
 else if (Sender==FontButton)
 {
  OpenDialogue->DefaultExt="fnt";
  OpenDialogue->FileName=M2000->FontImage;
  OpenDialogue->Filter="Font images|*.fnt";
 }
 else if (Sender==PrinterLogButton)
 {
  OpenDialogue->DefaultExt="prn";
  OpenDialogue->FileName=M2000->PrinterLog;
  OpenDialogue->Filter="Raw printer data|*.prn";
 }
 OpenDialogue->Filter+="|All files|*.*";
 OpenDialogue->FilterIndex=0;
 OpenDialogue->HistoryList->Clear();
 if (!OpenDialogue->Execute())
  return;
 if (Sender==M2000PathButton)
  M2000PathEdit->Text=M2000->M2000Path=OpenDialogue->FileName;
 else if (Sender==CassetteButton)
  CassetteEdit->Text=M2000->CassetteImage=OpenDialogue->FileName;
 else if (Sender==MonitorButton)
  MonitorEdit->Text=M2000->MonitorROM=OpenDialogue->FileName;
 else if (Sender==CartridgeButton)
  CartridgeEdit->Text=M2000->CartridgeROM=OpenDialogue->FileName;
 else if (Sender==FontButton)
  FontEdit->Text=M2000->FontImage=OpenDialogue->FileName;
 else if (Sender==PrinterLogButton)
  PrinterLogEdit->Text=M2000->PrinterLog=OpenDialogue->FileName;
}
//---------------------------------------------------------------------------
